;; This class is a cache-enabled version of its superclass ART Calibrating Marker Processor.
;; It offers the same data access methods as the superclass, but the processed marker data data 
;; are cached, so they can be retrieved much faster, if they have already been retrieved 
;; before. However, this caching consumes additional memory.
;; @field CACHED_MARKERS A pointer to cache the processed marker data retrieved. Or null this cache is not valid.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR__DEFINE

    ; define the class structure 
    struct = { CACHED_ART_CALIBRATING_MARKER_PROCESSOR, INHERITS art_calibrating_marker_processor, cached_markers:ptr_new() }

END


;; Does the same as the corresponding constructor of the superclass.
;;
FUNCTION CACHED_ART_CALIBRATING_MARKER_PROCESSOR::INIT, DATA, DISTANCE_THRESHOLD=DISTANCE_THRESHOLD, RELEVANT_MARKERS=RELEVANT_MARKERS, REFERENCE_BODY_ID=REFERENCE_BODY_ID, CALIBRATION_MATRIX=CALIBRATION_MATRIX

    ; initialize superclass
    return, self->art_calibrating_marker_processor::init(data, distance_threshold=distance_threshold, relevant_markers=relevant_markers, reference_body_id=reference_body_id, calibration_matrix=calibration_matrix)

END


;; Cleans up the heap upon destruction.
;; More specifically the superclass is cleaned up, and 
;; all cached markers are freed from the heap.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::CLEANUP

    self->art_calibrating_marker_processor::cleanup

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

END


;; Sets up the ART Camera Data the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::SET_ART_DATA, DATA

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::set_art_data, data

END


;; Sets up the distance threshold the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::SET_DISTANCE_THRESHOLD, DISTANCE_THRESHOLD

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::set_distance_threshold, distance_threshold

END


;; Sets up the relevant markers the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::SET_RELEVANT_MARKERS, RELEVANT_MARKERS

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::set_relevant_markers, relevant_markers

END


;; Finds the relevant markers the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::FIND_RELEVANT_MARKERS, MIN_SAMPLE_COUNT=MIN_SAMPLE_COUNT, MIN_SAMPLE_FRACTION=MIN_SAMPLE_FRACTION, MIN_SAMPLE_DURATION=MIN_SAMPLE_DURATION

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::find_relevant_markers, min_sample_count=min_sample_count, min_sample_fraction=min_sample_fraction, min_sample_duration=min_sample_duration

END


;; Sets up the reference body the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::SET_REFERENCE_BODY_ID, ID

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::set_reference_body_id, id

END


;; Sets up the calibration matrix the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::SET_CALIBRATION_MATRIX, CALIBRATION_MATRIX

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::set_calibration_matrix, calibration_matrix

END


;; Appends to the calibration matrix the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::APPEND_CALIBRATION_MATRIX, CALIBRATION_MATRIX

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::append_calibration_matrix, calibration_matrix

END


;; Loads the calibration matrix the same way as the corresponding method of the superclass.
;; Additionally, the cache of processed markers is invalidated.
;;
PRO CACHED_ART_CALIBRATING_MARKER_PROCESSOR::LOAD_CALIBRATION_MATRIX, FILENAME

    if ptr_valid(self.cached_markers) then begin
        heap_free, self.cached_markers
    endif

    self->art_calibrating_marker_processor::load_calibration_matrix, filename

END


;; Retrieves the processed markers, the same way the corresponding method of the superclass does.
;; Additionally some caching is applied. That is, when the markers are processed, they are also cached. 
;; The next requests for markers do not need to be processed again, as long as the setup of this
;; processor does not change. Instead, the processed markers can be drawn from the cache.
;;
FUNCTION CACHED_ART_CALIBRATING_MARKER_PROCESSOR::RETRIEVE_MARKERS

    ; update marker cache, if necessary
    if (~ptr_valid(self.cached_markers))  then begin
        markers = self->art_calibrating_marker_processor::retrieve_markers()
        self.cached_markers = ptr_new(markers)
    endif
    
    ; return cached markers
    return, *(self.cached_markers)

END
