;; Normalize an array of numeric data to a desired range. 
;; This is done by scanning the array for its minimum and maximum. Then all
;; values in the array are lineary transformed (i.e. scaled), so that the minimum 
;; and maximum fit the desired range exactly.
;; @param DATA_ARRAY (IN) The array to normalize.
;; @key MIN (?IN) The lower boundery of the desired range. Defaults to 0.
;; @key MAX (?IN) The upper boundary of the desired range. Defaults to 1.
;; @return A copy of the DATA_ARRAY with all values normalized to the given range.
;;
FUNCTION NORMALIZE, DATA_ARRAY, MIN=MIN, MAX=MAX

    ; use default min and max, if none are given
    if (n_elements(min) eq 0) then begin
        min=0.0
    endif
    if (n_elements(max) eq 0) then begin
        max=1.0
    endif

    ; calculate min and max of current data array
    old_min = min(data_array, max=old_max)

    ; transform data array to fit new min and max
    scale_factor = (max - min) / (old_max - old_min)
    offset = min - (old_min * scale_factor)
    return, offset + (scale_factor * data_array) 

END


;; Converts an integer number to a string using its natural length. 
;; The resulting string contains exactly the amount amount of characters 
;; needed to represent all digits of the number, and its sign, if appropriate.
;; @param NUMBER (IN) The number to convert.
;; @return A string representing the NUMBER.
;;
FUNCTION INT2STR, NUMBER

    return, string(number, format='(I0)')

END
